#Author- Gilles Muller
#Description- Save a set of sketches to DXF while removing construction lines

import adsk.core, adsk.fusion, traceback

app = None
ui  = None
commandId = 'DxfExport'
commandName = 'DxfExportSelect'
commandDescription = 'Select sketches for exportation'

# global set of event handlers to keep them referenced for the duration of the command
handlers = []
                    

def saveDXF(rootComp, directory, SK):
    newSketch = rootComp.sketches.add(SK.referencePlane)
    Matrix = adsk.core.Matrix3D.create()
    Curves = adsk.core.ObjectCollection.create()

    #for Curve in SK.sketchCurves:
    for s in range(len(SK.sketchCurves)):
        Curve = SK.sketchCurves[s]
        if not(Curve.isConstruction) :  
            Curves.add (Curve) 

    temp = SK.copy(Curves, Matrix, newSketch)
    BB = SK.boundingBox
         
    filename = directory + '/' + rootComp.name + '_' + SK.name + '.dxf'
    newSketch.saveAsDXF(filename)
    newSketch.deleteMe()
    return BB
   
def getSelectedObjects(selectionInput):
    objects = []

    for i in range(selectionInput.selectionCount):
        selection = selectionInput.selection(i)
        selectedObj = selection.entity

        if type(selectedObj) is adsk.fusion.Sketch:
            objects.append(selectedObj)
    return objects

 
class DxfExportInputChangedHandler(adsk.core.InputChangedEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            cmd = args.firingEvent.sender
            inputs = cmd.commandInputs
            appearanceListInput = None
            materialLibInput = None
            filterInput = None
            global commandId
            for inputI in inputs:
                if inputI.id == commandId + '_appearanceList':
                    appearanceListInput = inputI
                elif inputI.id == commandId + '_materialLib':
                    materialLibInput = inputI
                elif inputI.id == commandId + '_filter':
                    filterInput = inputI
            cmdInput = args.input

        except:
            if ui:
                ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

class DxfExportExecuteHandler(adsk.core.CommandEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            cmd = args.firingEvent.sender
            inputs = cmd.commandInputs
            selectionInput = None
            appearanceListInput = None
            for inputI in inputs:
                global commandId
                if inputI.id == commandId + '_selection':
                    selectionInput = inputI
            
            objects = getSelectedObjects(selectionInput)
            product = app.activeProduct
            design = adsk.fusion.Design.cast(product)
            rootComp = design.activeComponent

            # Set styles of file dialog.
            fldDialog = ui.createFolderDialog()
            fldDialog.title = 'Directory Selection'
            dlgResult = fldDialog.showDialog()

            deltax = 0
            deltay = 0

            if dlgResult == adsk.core.DialogResults.DialogOK:           
                for obj in objects:
                    BB = saveDXF(rootComp, fldDialog.folder, obj)
                    #ui.messageBox('Sketch:' + obj.name + '\n' +
                    #BB.maxPoint.x + BB.maxPoint.y + BB.maxPoint.z + '\n' + 
                    #BB.minPoint.x + BB.minPoint.y + BB.minPoint.z) 

                    BBInfo = "Sketch: %s\n" % (obj.name) 
                    BBInfo += "Max: %.4f, %.4f, %.4f\n" % (BB.maxPoint.x*10, BB.maxPoint.y*10, BB.maxPoint.z*10)
                    BBInfo += "Min: %.4f, %.4f, %.4f\n" % (BB.minPoint.x*10, BB.minPoint.y*10, BB.minPoint.z*10)
                    ui.messageBox(BBInfo, "Sketch Info")
                    
                    deltax = BB.minPoint.x - deltax
                    deltay = BB.minPoint.y - deltay
                
                deltaInfo = "Deltax: %.4f\n" % (abs(deltax*10))
                deltaInfo += "Deltay: %.4f\n" % (abs(deltay*10))
                ui.messageBox(deltaInfo, "Sketch Info")


        except:
            if ui:
                ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

class DxfExportDestroyHandler(adsk.core.CommandEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            # when the command is done, terminate the script
            # this will release all globals which will remove all event handlers
            adsk.terminate()
        except:
            if ui:
                ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

class DxfExportCreatedHandler(adsk.core.CommandCreatedEventHandler):    
    def __init__(self):
        super().__init__()        
    def notify(self, args):
        try:
            cmd = args.command
            cmd.isRepeatable = False
            onExecute = DxfExportExecuteHandler()
            cmd.execute.add(onExecute)
            
            onDestroy = DxfExportDestroyHandler()
            cmd.destroy.add(onDestroy)
            onInputChanged = DxfExportInputChangedHandler()
            cmd.inputChanged.add(onInputChanged)
            
            # keep the handler referenced beyond this function
            handlers.append(onExecute)
            handlers.append(onDestroy)
            handlers.append(onInputChanged)
            inputs = cmd.commandInputs
            global commandId
            selectionInput = inputs.addSelectionInput(commandId + '_selection', 'Select', 'Select sketches')
            selectionInput.setSelectionLimits(2)
         
        except:
            if ui:
                ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

def run(context):
    try:
        global app
        app = adsk.core.Application.get()
        global ui
        ui = app.userInterface

        global commandId
        global commandName
        global commandDescription
        
        cmdDef = ui.commandDefinitions.itemById(commandId)
        if not cmdDef:
            cmdDef = ui.commandDefinitions.addButtonDefinition(commandId, commandName, commandDescription) # no resource folder is specified, the default one will be used
     
        onCommandCreated = DxfExportCreatedHandler()
            
        cmdDef.commandCreated.add(onCommandCreated)
        # keep the handler referenced beyond this function
        handlers.append(onCommandCreated)

        inputs = adsk.core.NamedValues.create()
        cmdDef.execute(inputs)

        # prevent this module from being terminate when the script returns, because we are waiting for event handlers to fire
        adsk.autoTerminate(False)

    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))
